<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00001000ItemFormPrecoPropostaService;
use Teknisa\SM\Service\Sm00001000ItemLancamentoPropostaService;
use Teknisa\SM\Service\Sm00001000LancamentoPropostaService;

class Sm00001000ItemFormPrecoPropostaController extends SMCrud
{
    protected $dataSourceName = 'fct_relacestruturaprop_widget';
    protected $serviceItFormPrecProp;
    protected $itensPropostaService;
    protected $propostaService;

    public function __construct ($crudService, $environment, Sm00001000ItemFormPrecoPropostaService $serviceItFormPrecProp, Sm00001000ItemLancamentoPropostaService $itensPropostaService, Sm00001000LancamentoPropostaService $propostaService) {
        parent::__construct($crudService, $environment);
        $this->serviceItFormPrecProp = $serviceItFormPrecProp;
        $this->itensPropostaService  = $itensPropostaService;
        $this->propostaService       = $propostaService;
    }
    
    
    /**
     *  A função "insereItemFormPrec" organiza os dados para inserção da 
     *  formação de preço nos itens de contrato (feita no Service);
     */
    
    public function insereItemFormPrec(Request\Row $request, Response $response){
        $itensInseridos = array();
        try {
            $row = $request->getRow();
            
            $servicos        = $row["SERVICOS"];
            $soma            = $row["SOMA"];
            $proposta        = $row["PROPOSTA"];
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];

            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $itensInseridos = $this->serviceItFormPrecProp->insereItemFormPrec($servicos, $soma, $proposta, $nrtipoorcamento);
            foreach ($itensInseridos as $item) {
                $this->itensPropostaService->inserirItemNegociacaoValoresProposta($item, $nrorg, $cdoperador);
                
                if (isset($item["INSERIR_MEDIDA"]) && $item["INSERIR_MEDIDA"] == true) {
                    $this->propostaService->inserirOuAtualizarMedidaVariavelServico($item, $nrorg, $cdoperador);
                }
            }

            $response->addDataSet(new DataSet($this->dataSourceName, $itensInseridos));
        }
        catch (\Exception $e) {
            foreach ($itensInseridos as $item) {
                $this->itensPropostaService->excluirItemProposta($item["NRITEMORCAMENTO"], $nrorg);
            }
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * insere uma alteração da formação de preço
    **/
    public function insereAlteracaoFormPrec(Request\Row $request, Response $response){
        try {
            $row                = $request->getRow();
            $nrorg              = $this->environment->getNrOrg();
            $operador           = $this->environment->getCdOperador();
            $nrformprecoservico = $row["NRFORMPRECOSERVICO"];
            $nrseqorcamento     = $row["NRSEQORCAMENTO"];
            $nritemorcamento    = $row["NRITEMORCAMENTO"];
            
            $this->serviceItFormPrecProp->executarProcedureFctCopiaFormPrecoItProp($nritemorcamento, $nrseqorcamento, $nrformprecoservico, $nrorg, $operador);
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * insere uma alteração da formação de preço
    **/
    public function editaItensFormacaoAux(Request $request, Response $response){
        try {
            $rows     = $request->getDataSet()->getRows();
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $this->serviceItFormPrecProp->editaItensFormacaoAux($rows, $nrorg, $operador);
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta as funções que não foram utilizadas
    **/
    public function deletaAlteracoesInutilizadas(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrseqorcamento  = $row["NRSEQORCAMENTO"];
            $nritemorcamento = $row["NRITEMORCAMENTO"];
            $nrorg = $this->environment->getNrOrg();
            
            $this->serviceItFormPrecProp->deletaAlteracoesInutilizadas($nrseqorcamento, $nritemorcamento, $nrorg);
            if($nritemorcamento)
                $this->serviceItFormPrecProp->deletarRelacFormacaoProp($nrseqorcamento, $nritemorcamento, $nrorg);
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Deleta o item de formação de preço inserido quando a rotina de alteração do número 
     * do item na tabela de formção de preço auxiliar não funciona
    **/
    public function deletarItensFormacaoProp(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $nrseqorcamento  = $row["NRSEQORCAMENTO"];
            $nritemorcamento = $row["NRITEMORCAMENTO"];
            
            $this->serviceItFormPrecProp->deletarItensFormacaoProp($nrseqorcamento, $nritemorcamento, $nrorg);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * deleta a árvore completa de elementos alterados na formação de 
     * preço auxiliar já inserida na proposta
     **/
    public function deletarArvoreFormPrecoItPro(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $nrseqorcamento  = $row["NRSEQORCAMENTO"];
            $nritemorcamento = $row["NRITEMORCAMENTO"];
            $nrformprecoservico = $row["NRFORMPRECOSERVICO"];
            $nrcalcformprec     = $row["NRCALCFORMPRECSERV"];
            
            $this->serviceItFormPrecProp->deletarArvoreFormPrecoItPro($nrseqorcamento, $nritemorcamento, $nrorg, $nrformprecoservico, $nrcalcformprec);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * valida os dados de rateio do item
    **/
    public function validarDadosRateioProposta(Request\Row $request, Response $response) {
        try {
            $row     = $request->getRow();
            $nrorg   = $this->environment->getNrOrg();
            $rateios = null;
            
            $nrseqorcamento  = $row["NRSEQORCAMENTO"];
            $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
            
            $geraRateio = $this->serviceItFormPrecProp->getIdGeraRateioParametros($nrtipoorcamento, $nrorg);
            if($geraRateio[0]["IDGERARATEIO"] == 'N' && array_key_exists("NRESTRUTURAM", $row)) {
                $nrestruturam = $row["NRESTRUTURAM"];
                $rateios = $this->serviceItFormPrecProp->validarDadosRateioProposta($nrseqorcamento, $nrorg, $nrestruturam, $nrtipoorcamento);
            }
            
            $response->addDataSet(new DataSet($this->dataSourceName, array($rateios)));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}
?>